@@ -1,11 +1,13 @@
+/* ex: set tabstop=8 expandtab: */
 /*
-  $Id: cgn_tagger_mod.h 17992 2015-01-21 14:07:56Z sloot $
-  $URL: https://ilk.uvt.nl/svn/sources/Frog/trunk/include/frog/cgn_tagger_mod.h $
+  Copyright (c) 2006 - 2016
+  CLST  - Radboud University
+  ILK   - Tilburg University
 
-  Copyright (c) 2006 - 2015
-  Tilburg University
+  This file is part of frog:
 
-  This file is part of frog.
+  A Tagger-Lemmatizer-Morphological-Analyzer-Dependency-Parser for
+  several languages
 
   frog is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -14,40 +16,35 @@
 
   frog is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
-  
+
   For questions and suggestions, see:
-      http://ilk.uvt.nl/software.html
+      https://github.com/LanguageMachines/frog/issues
   or send mail to:
-      timbl@uvt.nl
+      lamasoftware (at ) science.ru.nl
+
 */
 
 #ifndef CGN_TAGGER_MOD_H
 #define CGN_TAGGER_MOD_H
 
-#include "mbt/MbtAPI.h"
+#include "frog/pos_tagger_mod.h"
 
-class CGNTagger {
+class CGNTagger: public POSTagger {
  public:
-  CGNTagger(TiCC::LogStream*);
-  ~CGNTagger();
+ CGNTagger( TiCC::LogStream *l ): POSTagger( l ){};
   bool init( const TiCC::Configuration& );
-  void addDeclaration( folia::Document& ) const;
-  void addTag( folia::Word *, const std::string&, double );
   void Classify( const std::vector<folia::Word *>& );
-  std::vector<TagResult> tagLine( const std::string& );
-  std::string getTagset() const { return tagset; };
+  void post_process( const std::vector<folia::Word *>& words );
  private:
-  MbtAPI *tagger;  
-  LogStream *cgnLog;
-  int debug;
-  std::string tagset;
-  std::string version;
-  Tokenizer::UnicodeFilter *filter;
+  void fillSubSetTable();
+  std::string getSubSet( const std::string& , const std::string& );
+  std::multimap<std::string,std::string> cgnSubSets;
+  std::multimap<std::string,std::string> cgnConstraints;
 };
 
 #endif // CGN_TAGGER_MOD_H
