@@ -1,11 +1,13 @@
+/* ex: set tabstop=8 expandtab: */
 /*
-  $Id: mblem_mod.h 17992 2015-01-21 14:07:56Z sloot $
-  $URL: https://ilk.uvt.nl/svn/sources/Frog/trunk/include/frog/mblem_mod.h $
+  Copyright (c) 2006 - 2016
+  CLST  - Radboud University
+  ILK   - Tilburg University
 
-  Copyright (c) 2006 - 2015
-  Tilburg University
+  This file is part of frog:
 
-  This file is part of frog.
+  A Tagger-Lemmatizer-Morphological-Analyzer-Dependency-Parser for
+  several languages
 
   frog is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -21,14 +23,14 @@
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
   For questions and suggestions, see:
-      http://ilk.uvt.nl/software.html
+      https://github.com/LanguageMachines/frog/issues
   or send mail to:
-      timbl@uvt.nl
+      lamasoftware (at ) science.ru.nl
+
 */
 
-#ifndef __MBLEM_MOD__
-#define __MBLEM_MOD__
-
+#ifndef MBLEM_MOD_H
+#define MBLEM_MOD_H
 
 class mblemData {
  public:
@@ -55,6 +57,8 @@ class Mblem {
   void filterTag( const std::string&  );
   void makeUnique();
   std::string getTagset() const { return tagset; };
+  bool fill_ts_map( const std::string& );
+  bool fill_eq_set( const std::string& );
  private:
   void read_transtable( const std::string& );
   void create_MBlem_defaults();
@@ -66,11 +70,14 @@ class Mblem {
   std::string punctuation;
   size_t history;
   int debug;
-  std::map <std::string,std::string> classMap;
+  bool keep_case;
+  std::map<std::string,std::string> classMap;
+  std::map<std::string, std::map<std::string, int>> token_strip_map;
+  std::set<std::string> one_one_tags;
   std::vector<mblemData> mblemResult;
   std::string version;
   std::string tagset;
-  std::string cgn_tagset;
+  std::string POS_tagset;
   TiCC::LogStream *mblemLog;
   Tokenizer::UnicodeFilter *filter;
 };
