@@ -1,9 +1,7 @@
 /*
-  $Id: ucto_tokenizer_mod.h 17992 2015-01-21 14:07:56Z sloot $
-  $URL: https://ilk.uvt.nl/svn/sources/Frog/trunk/include/frog/ucto_tokenizer_mod.h $
-
-  Copyright (c) 2006 - 2015
-  Tilburg University
+  Copyright (c) 2006 - 2016
+  CLST  - Radboud University
+  ILK   - Tilburg University
 
   This file is part of frog.
 
@@ -21,9 +19,10 @@
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
   For questions and suggestions, see:
-      http://ilk.uvt.nl/software.html
+      https://github.com/LanguageMachines/timblserver/issues
   or send mail to:
-      timbl@uvt.nl
+      lamasoftware (at ) science.ru.nl
+
 */
 
 #ifndef UCTO_TOKENIZER_MOD_H
@@ -34,7 +33,7 @@
 class UctoTokenizer {
  public:
   UctoTokenizer(TiCC::LogStream *);
-  ~UctoTokenizer() { delete tokenizer; delete uctoLog; };
+  ~UctoTokenizer() { delete tokenizer; };
   bool init( const TiCC::Configuration& );
   void setUttMarker( const std::string& );
   void setPassThru( bool );
@@ -45,10 +44,11 @@ class UctoTokenizer {
   void setInputXml( bool );
   void setTextClass( const std::string& );
   void setDocID( const std::string& );
-  folia::Document tokenizestring( const std::string& );
-  folia::Document tokenize( std::istream& );
+  folia::Document *tokenizestring( const std::string& );
+  folia::Document *tokenize( std::istream& );
   bool tokenize( folia::Document& );
   std::vector<std::string> tokenize( const std::string&  );
+  std::string tokenizeStream( std::istream& );
  private:
   Tokenizer::TokenizerClass *tokenizer;
   TiCC::LogStream *uctoLog;
