@@ -1,11 +1,13 @@
+/* ex: set tabstop=8 expandtab: */
 /*
-  $Id: Frog-util.cxx 17992 2015-01-21 14:07:56Z sloot $
-  $URL: https://ilk.uvt.nl/svn/sources/Frog/trunk/src/Frog-util.cxx $
+  Copyright (c) 2006 - 2016
+  CLST  - Radboud University
+  ILK   - Tilburg University
 
-  Copyright (c) 2006 - 2015
-  Tilburg University
+  This file is part of frog:
 
-  This file is part of frog
+  A Tagger-Lemmatizer-Morphological-Analyzer-Dependency-Parser for
+  several languages
 
   frog is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -21,9 +23,10 @@
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
   For questions and suggestions, see:
-      http://ilk.uvt.nl/software.html
+      https://github.com/LanguageMachines/frog/issues
   or send mail to:
-      timbl@uvt.nl
+      lamasoftware (at ) science.ru.nl
+
 */
 
 #include <set>
@@ -40,23 +43,15 @@
 using namespace std;
 
 string prefix( const string& path, const string& fn ){
-  if ( fn.find( "/" ) == string::npos ){
-    // only append prefix when NO path is specified
+  if ( fn.find( "/" ) == string::npos && !path.empty() ){
+    // only append prefix when it isn't empty AND
+    // NO path is specified in the filename
     return path + "/" + fn;
   }
   return fn;
 }
 
 #ifdef HAVE_DIRENT_H
-bool existsDir( const string& dirName ){
-  bool result = false;
-  DIR *dir = opendir( dirName.c_str() );
-  if ( dir ){
-    result = (closedir( dir ) == 0);
-  }
-  return result;
-}
-
 void getFileNames( const string& dirName,
 		   const string& ext,
 		   set<string>& fileNames ){
