@@ -1,14 +1,13 @@
 /* ex: set tabstop=8 expandtab: */
 /*
-  $Id: Frog.cxx 17992 2015-01-21 14:07:56Z sloot $
-  $URL: https://ilk.uvt.nl/svn/sources/Frog/trunk/src/Frog.cxx $
+  Copyright (c) 2006 - 2016
+  CLST  - Radboud University
+  ILK   - Tilburg University
 
-  Copyright (c) 2006 - 2015
-  Tilburg University
+  This file is part of frog:
 
-  A Tagger-Lemmatizer-Morphological-Analyzer-Dependency-Parser for Dutch
-
-  This file is part of frog
+  A Tagger-Lemmatizer-Morphological-Analyzer-Dependency-Parser for
+  several languages
 
   frog is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -24,14 +23,11 @@
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
   For questions and suggestions, see:
-      http://ilk.uvt.nl/software.html
+      https://github.com/LanguageMachines/frog/issues
   or send mail to:
-      timbl@uvt.nl
-*/
+      lamasoftware (at ) science.ru.nl
 
-// Python.h seems to best included first. It tramples upon defines like:
-// _XOPEN_SOURCE, _POSIX_C_SOURCE" etc.
-#include "Python.h"
+*/
 
 #include <cstdlib>
 #include <cstdio>
@@ -49,13 +45,15 @@
 #endif
 
 #include "timbl/TimblAPI.h"
-#include "timblserver/FdStream.h"
-#include "timblserver/ServerBase.h"
+#include "ticcutils/FdStream.h"
+#include "ticcutils/ServerBase.h"
 
 // individual module headers
 
 #include "frog/Frog.h" //internal interface, included by all modules
 #include "frog/FrogAPI.h" //public API interface
+#include "frog/ucto_tokenizer_mod.h"
+#include "frog/pos_tagger_mod.h"
 #include "ticcutils/StringOps.h"
 #include "ticcutils/CommandLine.h"
 #include "ticcutils/FileUtils.h"
@@ -104,7 +102,7 @@ void usage( ) {
        << "\t --uttmarker=<mark>     utterances are separated by 'mark' symbols"
        << "\t                        (default none)\n"
        << "\t -n                     Assume input file to hold one sentence per line\n"
-       << "\t --max-parser-tokens=<n> inhibit parsing when a sentence contains over 'n' tokens. (default: no limit)\n"
+       << "\t --max-parser-tokens=<n> inhibit parsing when a sentence contains over 'n' tokens. (default: 500, needs already 16Gb of memory!)\n"
        << "\t -Q                   Enable quote detection in tokeniser.\n"
        << "\t============= MODULE SELECTION ==========================================\n"
        << "\t --skip=[mptncla]    Skip Tokenizer (t), Lemmatizer (l), Morphological Analyzer (a), Chunker (c), Multi-Word Units (m), Named Entity Recognition (n), or Parser (p) \n"
@@ -117,40 +115,45 @@ void usage( ) {
        << "\t --id=<docid>          Document ID, used in FoLiA output. (Default 'untitled')\n"
        << "\t --outputdir=<dir>     Output to dir, instead of default stdout\n"
        << "\t --xmldir=<dir>        Use 'dir' to output FoliA XML to.\n"
-       << "\t --tmpdir=<directory>  (location to store intermediate files. Default /tmp )\n"
-       << "\t --keep-parser-files keep intermediate parser files. (last sentence only).\n"
+       << "\t --deep-morph          add deep morphological information to the output\n"
        << "\t============= OTHER OPTIONS ============================================\n"
-       << "\t -h. give some help.\n"
-       << "\t -V or --version .   Show version info.\n"
-       << "\t -d <debug level>    (for more verbosity)\n"
+       << "\t -h or --help          give some help.\n"
+       << "\t -V or --version       Show version info.\n"
+       << "\t -d <debug level>  (for more verbosity)\n"
        << "\t --debug=<module><level>,<module><level>... (eg --debug=l5,n3) \n"
        << "\t\t Set debug value for Tokenizer (t), Lemmatizer (l), Morphological Analyzer (a), Chunker (c), Multi-Word Units (m), Named Entity Recognition (n), or Parser (p) \n"
        << "\t -S <port>              Run as server instead of reading from testfile\n"
 #ifdef HAVE_OPENMP
-       << "\t --threads=<n>       Use a maximun of 'n' threads. Default: all we can get. \n"
+       << "\t --threads=<n>       Use a maximum of 'n' threads. Default: all we can get. \n"
 #endif
        << "\t                     (but always 1 for server mode)\n";
 }
 
-bool parse_args( TiCC::CL_Options& Opts, FrogOptions& options,
+bool parse_args( TiCC::CL_Options& Opts,
+		 FrogOptions& options,
 		 LogStream* theErrLog ) {
   // process the command line and fill FrogOptions to initialize the API
   // also fill some globals we use for our own main.
 
-  string value;
-  if ( Opts.is_present('V' ) || Opts.is_present("version" ) ){
-    // we already did show what we wanted.
-    exit( EXIT_SUCCESS );
+  // is a language specified? Default is dutch
+  string language = "nl";
+  Opts.extract ("language", language );
+  // is a config file for this language specified?
+  string configFileName = FrogAPI::defaultConfigFile(language);
+  if ( language == "nl" && !TiCC::isFile( configFileName ) ){
+    // for now, ignore language when it is dutch
+    language.clear();
+    configFileName = FrogAPI::defaultConfigFile();
   }
-  if ( Opts.is_present( 'h' ) ) {
-    usage();
-    exit( EXIT_SUCCESS );
-  };
-  // is a config file specified?
-  string configFileName = FrogAPI::defaultConfigFile();
+  options.language = language;
+  // override language settings when a configfile is specified
   Opts.extract( 'c',  configFileName );
   if ( configuration.fill( configFileName ) ){
     *Log(theErrLog) << "config read from: " << configFileName << endl;
+    string vers = configuration.lookUp( "version" );
+    if ( !vers.empty() ){
+      *Log(theErrLog) << "configuration version = " << vers << endl;
+    }
   }
   else {
     cerr << "failed to read configuration from '" << configFileName << "' !!" << endl;
@@ -158,6 +161,7 @@ bool parse_args( TiCC::CL_Options& Opts, FrogOptions& options,
     return false;
   }
 
+  string value;
   // debug opts
   if ( Opts.extract ('d', value) ) {
     if ( !stringTo<int>( value, options.debugFlag ) ){
@@ -173,9 +177,9 @@ bool parse_args( TiCC::CL_Options& Opts, FrogOptions& options,
     value = TiCC::lowercase( value );
     vector<string> vec;
     TiCC::split_at( value, vec, "," );
-    for ( size_t i=0; i < vec.size(); ++i ){
-      char mod = vec[i][0];
-      string value = vec[i].substr(1);
+    for ( const auto& val : vec ){
+      char mod = val[0];
+      string value = val.substr(1);
       int dbval = 0;
       if ( !stringTo<int>( value, dbval ) ){
 	cerr << "expected integer value for --debug=" << mod << value << endl;
@@ -235,10 +239,11 @@ bool parse_args( TiCC::CL_Options& Opts, FrogOptions& options,
     Opts.remove("skip");
   };
 
-  if ( Opts.extract( "daring" ) ) {
-    options.doDaringMorph = true;
+  if ( Opts.extract( "deep-morph" ) ) {
+    options.doDeepMorph = true;
     options.doMorph = true;
   }
+
   Opts.extract( 'e', options.encoding );
 
   if ( Opts.extract( "max-parser-tokens", value ) ){
@@ -254,6 +259,7 @@ bool parse_args( TiCC::CL_Options& Opts, FrogOptions& options,
   if ( options.doServer ) {
     // run in one thread in server mode, forking is too expensive for lots of small snippets
     options.numThreads =  1;
+    Opts.extract( "threads", value ); //discard threads option
   }
   else if ( Opts.extract( "threads", value ) ){
     int num;
@@ -263,47 +269,34 @@ bool parse_args( TiCC::CL_Options& Opts, FrogOptions& options,
     }
     options.numThreads = num;
   }
+#else
+  if ( Opts.extract( "threads", value ) ){
+    *Log(theErrLog) << "WARNING!\n---> There is NO OpenMP support enabled\n"
+		    << "---> --threads=" << value << " is ignored.\n"
+		    << "---> Will continue on just 1 thread." << endl;
+  }
 #endif
 
   if ( Opts.extract( "keep-parser-files" ) ){
-    configuration.setatt( "keepIntermediateFiles", "true", "parser" );
+    *Log(theErrLog) << "keep-parser-files option not longer supported. (ignored)" << endl;
   }
-  options.tmpDirName = configuration.lookUp( "tmpdir", "global" );
-  Opts.extract( "tmpdir", options.tmpDirName ); // so might be overridden
-  if ( options.tmpDirName.empty() ){
-    options.tmpDirName = "/tmp/";
+  if ( Opts.extract( "tmpdir" ) ){
+    *Log(theErrLog) << "tmpdir option not longer supported. (ignored)" << endl;
   }
-  else if ( options.tmpDirName[options.tmpDirName.size()-1] != '/' ){
-    options.tmpDirName += "/";
-  }
-#ifdef HAVE_DIRENT_H
-  if ( !options.tmpDirName.empty() ){
-    if ( !existsDir( options.tmpDirName ) ){
-      *Log(theErrLog) << "temporary dir " << options.tmpDirName << " not readable" << endl;
-      return false;
-    }
-    *Log(theErrLog) << "checking tmpdir: " << options.tmpDirName << " OK" << endl;
-  }
-#endif
   string TestFileName;
   if ( Opts.extract( "testdir", TestFileName ) ) {
-#ifdef HAVE_DIRENT_H
     options.doDirTest = true;
     testDirName = TestFileName;
     if ( testDirName[testDirName.size()-1] != '/' ){
       testDirName += "/";
     }
-    if ( !existsDir( testDirName ) ){
+    if ( !TiCC::isDir( testDirName ) ){
       *Log(theErrLog) << "input dir " << testDirName << " not readable" << endl;
       return false;
     }
-#else
-      *Log(theErrLog) << "--testdir option not supported!" << endl;
-#endif
   }
   else if ( Opts.extract( 't', TestFileName ) ) {
-    ifstream is( TestFileName );
-    if ( !is ){
+    if ( !TiCC::isFile( TestFileName ) ){
       *Log(theErrLog) << "input stream " << TestFileName << " is not readable" << endl;
       return false;
     }
@@ -313,12 +306,10 @@ bool parse_args( TiCC::CL_Options& Opts, FrogOptions& options,
     if ( outputDirName[outputDirName.size()-1] != '/' ){
       outputDirName += "/";
     }
-#ifdef HAVE_DIRENT_H
-    if ( !existsDir( outputDirName ) ){
+    if ( !TiCC::createPath( outputDirName ) ){
       *Log(theErrLog) << "output dir " << outputDirName << " not readable" << endl;
       return false;
     }
-#endif
     wantOUT = true;
   }
   else if ( Opts.extract( 'o', outputFileName ) ){
@@ -330,12 +321,10 @@ bool parse_args( TiCC::CL_Options& Opts, FrogOptions& options,
     if ( xmlDirName[xmlDirName.size()-1] != '/' ){
       xmlDirName += "/";
     }
-#ifdef HAVE_DIRENT_H
-    if ( !existsDir( xmlDirName ) ){
+    if ( !TiCC::createPath( xmlDirName ) ){
       *Log(theErrLog) << "XML output dir " << xmlDirName << " not readable" << endl;
       return false;
     }
-#endif
     options.doXMLout = true;
   }
   else if ( Opts.extract('X', XMLoutFileName ) ){
@@ -348,15 +337,12 @@ bool parse_args( TiCC::CL_Options& Opts, FrogOptions& options,
   if ( Opts.extract ('x', value ) ){
     options.doXMLin = true;
     if ( !value.empty() ){
-      if ( ! (xmlDirName.empty() &&
-	      testDirName.empty() &&
-	      TestFileName.empty() ) ){
+      if ( !xmlDirName.empty() || !testDirName.empty() ){
 	*Log(theErrLog) << "-x may not provide a value when --testdir or --xmldir is provided" << endl;
 	return false;
       }
       TestFileName = value;
-      ifstream is( value.c_str() );
-      if ( !is ){
+      if ( !TiCC::isFile( TestFileName ) ){
 	*Log(theErrLog) << "input stream " << value << " is not readable" << endl;
 	return false;
       }
@@ -375,10 +361,6 @@ bool parse_args( TiCC::CL_Options& Opts, FrogOptions& options,
   }
 
   Opts.extract ("uttmarker", options.uttmark );
-  if ( !outputDirName.empty() && testDirName.empty() ){
-    *Log(theErrLog) << "useless -outputdir option" << endl;
-    return false;
-  }
   if ( !testDirName.empty() ){
     if ( options.doXMLin )
       getFileNames( testDirName, ".xml", fileNames );
@@ -396,8 +378,8 @@ bool parse_args( TiCC::CL_Options& Opts, FrogOptions& options,
     }
     else {
       vector<string> mass = Opts.getMassOpts();
-      for (size_t i=0; i < mass.size(); ++i ){
-	fileNames.insert( mass[i] );
+      for ( const auto& name : mass ){
+	fileNames.insert( name );
       }
     }
   }
@@ -416,10 +398,23 @@ bool parse_args( TiCC::CL_Options& Opts, FrogOptions& options,
   return true;
 }
 
+bool StillRunning = true;
+
+void KillServerFun( int Signal ){
+  if ( Signal == SIGTERM ){
+    cerr << "KillServerFun caught a signal SIGTERM" << endl;
+    sleep(5); // give children some spare time...
+    StillRunning = false;
+  }
+}
+
 
 int main(int argc, char *argv[]) {
-  cerr << "frog " << VERSION << " (c) ILK 1998 - 2015" << endl;
-  cerr << "Induction of Linguistic Knowledge Research Group, Tilburg University" << endl;
+  cerr << "frog " << VERSION << " (c) CLTS, ILK 1998 - 2016" << endl
+       << "CLST  - Centre for Language and Speech Technology,"
+       << "Radboud University" << endl
+       << "ILK   - Induction of Linguistic Knowledge Research Group,"
+       << "Tilburg University" << endl;
   cerr << "based on [" << Tokenizer::VersionName() << ", "
        << folia::VersionName() << ", "
        << Timbl::VersionName() << ", "
@@ -432,10 +427,19 @@ int main(int argc, char *argv[]) {
   try {
     TiCC::CL_Options Opts("c:e:o:t:x::X::nQhVd:S:",
 			  "textclass:,testdir:,uttmarker:,max-parser-tokens:,"
-			  "skip:,id:,outputdir:,xmldir:,tmpdir:,daring,"
+			  "skip:,id:,outputdir:,xmldir:,tmpdir:,deep-morph,"
+			  "help,language:,"
 			  "debug:,keep-parser-files,version,threads:,KANON");
-
     Opts.init(argc, argv);
+    if ( Opts.is_present('V' ) || Opts.is_present("version" ) ){
+      // we already did show what we wanted.
+      return EXIT_SUCCESS;
+    }
+    if ( Opts.is_present( 'h' )
+	 || Opts.is_present( "help" ) ) {
+      usage();
+      return EXIT_SUCCESS;
+    };
     bool parsed = parse_args( Opts, options, theErrLog );
     if (!parsed) {
       throw runtime_error( "init failed" );
@@ -445,18 +449,21 @@ int main(int argc, char *argv[]) {
     if ( !fileNames.empty() ) {
       string outPath = outputDirName;
       string xmlPath = xmlDirName;
-      set<string>::const_iterator it = fileNames.begin();
+
       ostream *outS = 0;
       if ( !outputFileName.empty() ){
-        outS = new ofstream( outputFileName.c_str() );
+	if ( !TiCC::createPath( outputFileName ) ) {
+	  *Log(theErrLog) << "problem: unable to create outputfile: "
+			  << outputFileName << endl;
+	  return EXIT_FAILURE;
+	}
+        outS = new ofstream( outputFileName );
       }
-      while ( it != fileNames.end() ){
-	string testName = testDirName;
-	testName += *it;
+      for ( auto const& name : fileNames ){
+	string testName = testDirName + name;
 	if ( !TiCC::isFile( testName ) ){
 	  *Log(theErrLog) << "skip " << testName << " (file not found )"
 			  << endl;
-	  ++it;
 	  continue;
 	}
 	string outName;
@@ -464,35 +471,53 @@ int main(int argc, char *argv[]) {
 	  if ( wantOUT ){
 	    if ( options.doXMLin ){
 	      if ( !outPath.empty() )
-		outName = outPath + *it + ".out";
+		outName = outPath + name + ".out";
 	    }
 	    else {
-	      outName = outPath + *it + ".out";
+	      outName = outPath + name + ".out";
 	    }
-	    outS = new ofstream( outName.c_str() );
+	    if ( !TiCC::createPath( outName ) ) {
+	      *Log(theErrLog) << "problem frogging: " << name << endl
+			      << "unable to create outputfile: " << outName
+			      << endl;
+	      continue;
+	    }
+	    outS = new ofstream( outName );
 	  } else {
 	    outS = &cout;
 	  }
 	}
-	string xmlName = XMLoutFileName;
-	if ( xmlName.empty() ){
+	string xmlOutName = XMLoutFileName;
+	if ( xmlOutName.empty() ){
 	  if ( !xmlDirName.empty() ){
-	    if ( it->rfind(".xml") == string::npos )
-	      xmlName = xmlPath + *it + ".xml";
+	    if ( name.rfind(".xml") == string::npos )
+	      xmlOutName = xmlPath + name + ".xml";
 	    else
-	      xmlName = xmlPath + *it;
+	      xmlOutName = xmlPath + name;
 	  }
 	  else if ( options.doXMLout )
-	    xmlName = *it + ".xml"; // do not clobber the inputdir!
+	    xmlOutName = name + ".xml"; // do not clobber the inputdir!
+	}
+	if ( !xmlOutName.empty() && !TiCC::createPath( xmlOutName ) ){
+	  *Log(theErrLog) << "problem frogging: " << name << endl
+			  << "unable to create outputfile: " << xmlOutName
+			  << endl;
+	  continue;
 	}
 	*Log(theErrLog) << "Frogging " << testName << endl;
-	frog.FrogFile( testName, *outS, xmlName );
+	try {
+	  frog.FrogFile( testName, *outS, xmlOutName );
+	}
+	catch ( exception& e ){
+	  *Log(theErrLog) << "problem frogging: " << name << endl
+			  << e.what() << endl;
+	  continue;
+	}
 	if ( !outName.empty() ){
 	  *Log(theErrLog) << "results stored in " << outName << endl;
 	  delete outS;
 	  outS = 0;
 	}
-	++it;
       }
       if ( !outputFileName.empty() ){
 	*Log(theErrLog) << "results stored in " << outputFileName << endl;
@@ -508,6 +533,11 @@ int main(int argc, char *argv[]) {
       action.sa_flags = SA_NOCLDWAIT;
 #endif
       sigaction(SIGCHLD, &action, NULL);
+      struct sigaction act;
+      sigaction( SIGTERM, NULL, &act ); // get current action
+      act.sa_handler = KillServerFun;
+      act.sa_flags &= ~SA_RESTART;      // do not continue after SIGTERM
+      sigaction( SIGTERM, &act, NULL );
 
       srand((unsigned)time(0));
 
@@ -522,18 +552,19 @@ int main(int argc, char *argv[]) {
 	  // maximum of 5 pending requests
 	  throw( runtime_error( "listen(5) failed" ) );
 	}
-	while ( true ) {
+	while ( StillRunning ) {
 	  Sockets::ServerSocket conn;
 	  if ( server.accept( conn ) ){
 	    *Log(theErrLog) << "New connection..." << endl;
 	    int pid = fork();
 	    if (pid < 0) {
-	      *Log(theErrLog) << "ERROR on fork" << endl;
-	      throw runtime_error( "FORK failed" );
+	      string err = strerror(errno);
+	      *Log(theErrLog) << "ERROR on fork: " << err << endl;
+	      throw runtime_error( "FORK failed: " + err );
 	    }
 	    else if (pid == 0)  {
 	      frog.FrogServer( conn );
-	      exit(EXIT_SUCCESS);
+	      return EXIT_SUCCESS;
 	    }
 	  }
 	  else {
@@ -555,5 +586,6 @@ int main(int argc, char *argv[]) {
     *Log(theErrLog) << "fatal error: " << e.what() << endl;
     return EXIT_FAILURE;
   }
+  delete theErrLog;
   return EXIT_SUCCESS;
 }
