@@ -1,13 +1,13 @@
+/* ex: set tabstop=8 expandtab: */
 /*
-  $Id: iob_tagger_mod.cxx 17992 2015-01-21 14:07:56Z sloot $
-  $URL: https://ilk.uvt.nl/svn/sources/Frog/trunk/src/iob_tagger_mod.cxx $
+  Copyright (c) 2006 - 2016
+  CLST  - Radboud University
+  ILK   - Tilburg University
 
-  Copyright (c) 2006 - 2015
-  Tilburg University
+  This file is part of frog:
 
-  A Tagger-Lemmatizer-Morphological-Analyzer-Dependency-Parser for Dutch
-
-  This file is part of frog
+  A Tagger-Lemmatizer-Morphological-Analyzer-Dependency-Parser for
+  several languages
 
   frog is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -23,26 +23,32 @@
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
   For questions and suggestions, see:
-      http://ilk.uvt.nl/software.html
+      https://github.com/LanguageMachines/frog/issues
   or send mail to:
-      timbl@uvt.nl
+      lamasoftware (at ) science.ru.nl
+
 */
 
 #include "mbt/MbtAPI.h"
 #include "frog/Frog.h"
+#include "ucto/unicode.h"
 #include "frog/iob_tagger_mod.h"
 
 using namespace std;
 using namespace folia;
+using namespace TiCC;
+using namespace Tagger;
 
 IOBTagger::IOBTagger(TiCC::LogStream * logstream){
   tagger = 0;
   iobLog = new LogStream( logstream, "iob-" );
+  filter = 0;
 }
 
 IOBTagger::~IOBTagger(){
   delete tagger;
   delete iobLog;
+  delete filter;
 }
 
 bool IOBTagger::init( const Configuration& config ){
@@ -72,8 +78,9 @@ bool IOBTagger::init( const Configuration& config ){
   default:
     iobLog->setlevel(LogExtreme);
   }
-  if (debug)
+  if (debug) {
     *Log(iobLog) << "IOB Chunker Init" << endl;
+  }
   if ( tagger != 0 ){
     *Log(iobLog) << "IOBTagger is already initialized!" << endl;
     return false;
@@ -84,24 +91,36 @@ bool IOBTagger::init( const Configuration& config ){
     return false;
   }
   string settings;
-  if ( val[0] == '/' ) // an absolute path
+  if ( val[0] == '/' ) {
+    // an absolute path
     settings = val;
-  else
+  }
+  else {
     settings = config.configDir() + val;
+  }
 
   val = config.lookUp( "version", "IOB" );
   if ( val.empty() ){
     version = "1.0";
   }
-  else
+  else {
     version = val;
+  }
   val = config.lookUp( "set", "IOB" );
   if ( val.empty() ){
     tagset = "http://ilk.uvt.nl/folia/sets/frog-chunker-nl";
   }
-  else
+  else {
     tagset = val;
-
+  }
+  string charFile = config.lookUp( "char_filter_file", "IOB" );
+  if ( charFile.empty() )
+    charFile = config.lookUp( "char_filter_file" );
+  if ( !charFile.empty() ){
+    charFile = prefix( config.configDir(), charFile );
+    filter = new Tokenizer::UnicodeFilter();
+    filter->fill( charFile );
+  }
   string init = "-s " + settings + " -vcf";
   tagger = new MbtAPI( init, *iobLog );
   return tagger->isInit();
@@ -112,8 +131,8 @@ void IOBTagger::addChunk( ChunkingLayer *chunks,
 			  const vector<double>& confs,
 			  const string& IOB ){
   double conf = 1;
-  for ( size_t i=0; i < confs.size(); ++i )
-    conf *= confs[i];
+  for ( auto const& val : confs )
+    conf *= val;
   KWargs args;
   args["class"] = IOB;
   args["set"] = tagset;
@@ -123,7 +142,7 @@ void IOBTagger::addChunk( ChunkingLayer *chunks,
 #pragma omp critical(foliaupdate)
   {
     try {
-      chunk = new Chunk( chunks->doc(), args );
+      chunk = new Chunk( args, chunks->doc() );
       chunks->append( chunk );
     }
     catch ( exception& e ){
@@ -131,12 +150,13 @@ void IOBTagger::addChunk( ChunkingLayer *chunks,
       exit( EXIT_FAILURE );
     }
   }
-  for ( size_t i=0; i < words.size(); ++i ){
-    if ( words[i]->isinstance(PlaceHolder_t) )
+  for ( const auto& word : words ){
+    if ( word->isinstance(PlaceHolder_t) ){
       continue;
+    }
 #pragma omp critical(foliaupdate)
     {
-      chunk->append( words[i] );
+      chunk->append( word );
     }
   }
 }
@@ -144,8 +164,9 @@ void IOBTagger::addChunk( ChunkingLayer *chunks,
 void IOBTagger::addIOBTags( const vector<Word*>& words,
 			    const vector<string>& tags,
 			    const vector<double>& confs ){
-  if ( words.empty() )
+  if ( words.empty() ){
     return;
+  }
   ChunkingLayer *el = 0;
 #pragma omp critical(foliaupdate)
   {
@@ -157,7 +178,7 @@ void IOBTagger::addIOBTags( const vector<Word*>& words,
       KWargs args;
       args["generate_id"] = sent->id();
       args["set"] = tagset;
-      el = new ChunkingLayer(sent->doc(),args);
+      el = new ChunkingLayer( args, sent->doc() );
       sent->append( el );
     }
   }
@@ -165,8 +186,9 @@ void IOBTagger::addIOBTags( const vector<Word*>& words,
   vector<double> dstack;
   string curIOB;
   for ( size_t i=0; i < tags.size(); ++i ){
-    if (debug)
+    if (debug){
       *Log(iobLog) << "tag = " << tags[i] << endl;
+    }
     vector<string> tagwords;
     size_t num_words = TiCC::split_at( tags[i], tagwords, "_" );
     if ( num_words != 2 ){
@@ -174,8 +196,9 @@ void IOBTagger::addIOBTags( const vector<Word*>& words,
       exit( EXIT_FAILURE );
     }
     vector<string> iob;
-    if (debug)
+    if (debug){
       *Log(iobLog) << "IOB = " << tagwords[1] << endl;
+    }
     if ( tagwords[1] == "O" ){
       if ( !stack.empty() ){
 	if (debug) {
@@ -227,24 +250,34 @@ void IOBTagger::addIOBTags( const vector<Word*>& words,
 }
 
 void IOBTagger::addDeclaration( Document& doc ) const {
-  doc.declare( AnnotationType::CHUNKING,
-	       tagset,
-	       "annotator='frog-chunker-" + version
-	       + "', annotatortype='auto', datetime='" + getTime() + "'");
+#pragma omp critical(foliaupdate)
+  {
+    doc.declare( AnnotationType::CHUNKING,
+		 tagset,
+		 "annotator='frog-chunker-" + version
+		 + "', annotatortype='auto', datetime='" + getTime() + "'");
+  }
 }
 
 void IOBTagger::Classify( const vector<Word *>& swords ){
   if ( !swords.empty() ) {
-    vector<string> words;
     string sentence; // the tagger needs the whole sentence
-    for ( size_t w = 0; w < swords.size(); ++w ) {
-      sentence += swords[w]->str();
-      words.push_back( swords[w]->str() );
-      if ( w < swords.size()-1 )
+    for ( const auto& sword : swords ){
+      UnicodeString word;
+#pragma omp critical(foliaupdate)
+      {
+	word = sword->text();
+      }
+      if ( filter )
+	word = filter->filter( word );
+      sentence += UnicodeToUTF8(word);
+      if ( &sword != &swords.back() ){
 	sentence += " ";
+      }
     }
-    if (debug)
+    if (debug){
       *Log(iobLog) << "IOB in: " << sentence << endl;
+    }
     vector<TagResult> tagv = tagger->TagLine(sentence);
     if ( tagv.size() != swords.size() ){
       throw runtime_error( "IOB tagger is confused" );
@@ -259,10 +292,19 @@ void IOBTagger::Classify( const vector<Word *>& swords ){
     }
     vector<double> conf;
     vector<string> tags;
-    for ( size_t i=0; i < tagv.size(); ++i ){
-      tags.push_back( tagv[i].assignedTag() );
-      conf.push_back( tagv[i].confidence() );
+    for ( const auto& tag : tagv ){
+      tags.push_back( tag.assignedTag() );
+      conf.push_back( tag.confidence() );
     }
     addIOBTags( swords, tags, conf );
   }
 }
+
+string IOBTagger::set_eos_mark( const string& eos ){
+  if ( tagger ){
+    return tagger->set_eos_mark(eos);
+  }
+  else {
+    throw runtime_error( "IOBTagger is not initialized" );
+  }
+}
