@@ -1,11 +1,13 @@
+/* ex: set tabstop=8 expandtab: */
 /*
-  $Id: mblem_mod.cxx 17992 2015-01-21 14:07:56Z sloot $
-  $URL: https://ilk.uvt.nl/svn/sources/Frog/trunk/src/mblem_mod.cxx $
+  Copyright (c) 2006 - 2016
+  CLST  - Radboud University
+  ILK   - Tilburg University
 
-  Copyright (c) 2006 - 2015
-  Tilburg University
+  This file is part of frog:
 
-  This file is part of frog.
+  A Tagger-Lemmatizer-Morphological-Analyzer-Dependency-Parser for
+  several languages
 
   frog is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -21,9 +23,10 @@
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
   For questions and suggestions, see:
-      http://ilk.uvt.nl/software.html
+      https://github.com/LanguageMachines/frog/issues
   or send mail to:
-      timbl@uvt.nl
+      lamasoftware (at ) science.ru.nl
+
 */
 
 #include <cstdlib>
@@ -45,35 +48,34 @@ Mblem::Mblem( LogStream *logstream ):
   myLex(0),
   punctuation( "?...,:;\\'`(){}[]%#+-_=/!" ),
   history(20),
-  debug(0)
+  debug(0),
+  keep_case( false ),
+  filter(0)
 {
   mblemLog = new LogStream( logstream, "mblem" );
 }
 
-void Mblem::read_transtable( const string& tableName ) {
-  ifstream bron( tableName.c_str() );
-  if ( !bron ) {
-    *Log(mblemLog) << "translation table file '" << tableName
-		    << "' appears to be missing." << endl;
-    exit(1);
+bool Mblem::fill_ts_map( const string& file ){
+  ifstream is( file );
+  if ( !is ){
+    *Log(mblemLog) << "Unable to open file: '" << file << "'" << endl;
+    return false;
   }
-  while( bron ){
-    string className;
-    string classCode;
-    bron >> className;
-    bron >> ws;
-    bron >> classCode;
-    if ( classMap.find( classCode ) == classMap.end() ){
-      // stupid HACK to only accept first occurence
-      // multiple occurences is a NO NO i think
-      classMap[classCode] = className;
+  string line;
+  while ( getline( is, line ) ){
+    if ( line.empty() || line[0] == '#' )
+      continue;
+    vector<string> parts;
+    if ( TiCC::split( line, parts ) != 3 ){
+      *Log(mblemLog) << "invalid line in: '" << file << "' (expected 3 parts)" << endl;
+      return false;
     }
-    // else {
-    //   *Log(mblemLog) << "multiple entry " << className << " " << classCode << " in translation table file: " << tableName  << " (Ignored) " << endl;
-    // }
-    bron >> ws;
+    token_strip_map[parts[0]].insert( make_pair( parts[1], TiCC::stringTo<int>( parts[2] ) ) );
   }
-  return;
+  if ( debug ){
+    *Log(mblemLog) << "read token strip rules from: '" << file << "'" << endl;
+  }
+  return true;
 }
 
 bool Mblem::init( const Configuration& config ) {
@@ -101,15 +103,16 @@ bool Mblem::init( const Configuration& config ) {
 
   val = config.lookUp( "set", "tagger" );
   if ( val.empty() ){
-    cgn_tagset = "http://ilk.uvt.nl/folia/sets/frog-mbpos-cgn";
+    POS_tagset = "http://ilk.uvt.nl/folia/sets/frog-mbpos-cgn";
   }
   else
-    cgn_tagset = val;
+    POS_tagset = val;
 
   string transName = config.lookUp( "transFile", "mblem" );
   if ( !transName.empty() ){
     transName = prefix( config.configDir(), transName );
-    read_transtable( transName );
+    cerr << "usage of a mblem transFile is no longer needed!" << endl;
+    cerr << "skipping : " << transName << endl;
   }
   string treeName = config.lookUp( "treeFile", "mblem"  );
   if ( treeName.empty() )
@@ -125,6 +128,27 @@ bool Mblem::init( const Configuration& config ) {
     filter->fill( charFile );
   }
 
+  string tokenStripFile = config.lookUp( "token_strip_file", "mblem" );
+  if ( !tokenStripFile.empty() ){
+    tokenStripFile = prefix( config.configDir(), tokenStripFile );
+    if ( !fill_ts_map( tokenStripFile ) )
+      return false;
+  }
+
+  string one_one_tagS = config.lookUp( "one_one_tags", "mblem" );
+  if ( !one_one_tagS.empty() ){
+    vector<string> tags;
+    TiCC::split_at( one_one_tagS, tags, "," );
+    for ( auto const& t : tags ){
+      one_one_tags.insert( t );
+    }
+  }
+
+  string par = config.lookUp( "keep_case", "mblem" );
+  if ( !par.empty() ){
+    keep_case = stringTo<bool>( par );
+  }
+
   string opts = config.lookUp( "timblOpts", "mblem" );
   if ( opts.empty() )
     opts = "-a1";
@@ -166,43 +190,10 @@ string Mblem::make_instance( const UnicodeString& in ) {
   return result;
 }
 
-bool similar( const string& tag, const string& lookuptag,
-	      const string& CGNentry ){
-  return tag.find( CGNentry ) != string::npos &&
-    lookuptag.find( CGNentry ) != string::npos ;
-}
-
-bool isSimilar( const string& tag, const string& cgnTag ){
-  // Dutch CGN constraints
-  return
-    tag == cgnTag ||
-    similar( tag, cgnTag, "hulpofkopp" ) ||
-    similar( tag, cgnTag, "neut,zelfst" ) ||
-    similar( tag, cgnTag, "rang,bep,zelfst,onverv" ) ||
-    similar( tag, cgnTag, "stell,onverv" ) ||
-    similar( tag, cgnTag, "hoofd,prenom" ) ||
-    similar( tag, cgnTag, "soort,ev" ) ||
-    similar( tag, cgnTag, "ev,neut" ) ||
-    similar( tag, cgnTag, "inf" ) ||
-    similar( tag, cgnTag, "zelfst" ) ||
-    similar( tag, cgnTag, "voorinf" ) ||
-    similar( tag, cgnTag, "verldw,onverv" ) ||
-    similar( tag, cgnTag, "ott,3,ev" ) ||
-    similar( tag, cgnTag, "ott,2,ev" ) ||
-    similar( tag, cgnTag, "ott,1,ev" ) ||
-    similar( tag, cgnTag, "ott,1of2of3,mv" ) ||
-    similar( tag, cgnTag, "ott,1of2of3,ev" ) ||
-    similar( tag, cgnTag, "ovt,1of2of3,mv" ) ||
-    similar( tag, cgnTag, "ovt,1of2of3,ev" ) ||
-    similar( tag, cgnTag, "ovt,3,ev" ) ||
-    similar( tag, cgnTag, "ovt,2,ev" ) ||
-    similar( tag, cgnTag, "ovt,1,ev" );
-}
-
 void Mblem::addLemma( Word *word, const string& cls ){
   KWargs args;
   args["set"]=tagset;
-  args["cls"]=cls;
+  args["class"]=cls;
 #pragma omp critical(foliaupdate)
   {
     try {
@@ -216,18 +207,21 @@ void Mblem::addLemma( Word *word, const string& cls ){
 }
 
 void Mblem::filterTag( const string& postag ){
-  vector<mblemData>::iterator it = mblemResult.begin();
+  auto it = mblemResult.begin();
   while( it != mblemResult.end() ){
-    if ( isSimilar( postag, it->getTag() ) ){
-      if ( debug )
-	*Log(mblemLog) << "compare cgn-tag " << postag << " with "
-		       << it->getTag() << " similar! " << endl;
+    string tag = it->getTag();
+    if ( postag == tag ){
+      if ( debug ){
+	*Log(mblemLog) << "compare cgn-tag " << postag << " with mblem-tag " << tag
+		       << "\n\t==> identical tags"  << endl;
+      }
       ++it;
     }
     else {
-      if ( debug )
-	*Log(mblemLog) << "compare cgn-tag " << postag << " with "
-		       << it->getTag() << " NOT similar! " << endl;
+      if ( debug ){
+	*Log(mblemLog) << "compare cgn-tag " << postag << " with mblem-tag " << tag
+		       << "\n\t==> different tags" << endl;
+      }
       it = mblemResult.erase(it);
     }
   }
@@ -236,10 +230,10 @@ void Mblem::filterTag( const string& postag ){
 }
 
 void Mblem::makeUnique( ){
-  vector<mblemData>::iterator it = mblemResult.begin();
+  auto it = mblemResult.begin();
   while( it != mblemResult.end() ){
     string lemma = it->getLemma();
-    vector<mblemData>::iterator it2 = it+1;
+    auto it2 = it+1;
     while( it2 != mblemResult.end() ){
       if (debug)
 	*Log(mblemLog) << "compare lemma " << lemma << " with " << it2->getLemma() << " ";
@@ -258,9 +252,9 @@ void Mblem::makeUnique( ){
   }
   if (debug){
     *Log(mblemLog) << "final result after filter and unique" << endl;
-    for( size_t index=0; index < mblemResult.size(); ++index ){
-      *Log(mblemLog) << "lemma alt: " << mblemResult[index].getLemma()
-		     << "\ttag alt: " << mblemResult[index].getTag() << endl;
+    for ( const auto& mbr : mblemResult ){
+      *Log(mblemLog) << "lemma alt: " << mbr.getLemma()
+		     << "\ttag alt: " << mbr.getTag() << endl;
     }
   }
 }
@@ -272,69 +266,74 @@ void Mblem::getFoLiAResult( Word *word, const UnicodeString& uWord ){
     addLemma( word, result );
   }
   else {
-    vector<mblemData>::iterator it = mblemResult.begin();
-    while( it != mblemResult.end() ){
-      string result = it->getLemma();
+    for ( auto const& it : mblemResult ){
+      string result = it.getLemma();
       addLemma( word, result );
-      ++it;
     }
   }
 }
 
 
 void Mblem::addDeclaration( Document& doc ) const {
-  doc.declare( AnnotationType::LEMMA,
-	       tagset,
-	       "annotator='frog-mblem-" + version
-	       + "', annotatortype='auto', datetime='" + getTime() + "'");
+#pragma omp critical (foliaupdate)
+  {
+    doc.declare( AnnotationType::LEMMA,
+		 tagset,
+		 "annotator='frog-mblem-" + version
+		 + "', annotatortype='auto', datetime='" + getTime() + "'");
+  }
 }
 
 void Mblem::Classify( Word *sword ){
   if ( sword->isinstance(PlaceHolder_t ) )
     return;
-  UnicodeString word;
-  string tag;
+  UnicodeString uword;
   string pos;
   string token_class;
 #pragma omp critical(foliaupdate)
   {
-    word = sword->text();
+    uword = sword->text();
     pos = sword->pos();
     token_class = sword->cls();
-    tag = sword->annotation<PosAnnotation>( cgn_tagset )->feat("head");
   }
   if (debug)
-    *Log(mblemLog) << "Classify " << word << "(" << pos << ") ["
+    *Log(mblemLog) << "Classify " << uword << "(" << pos << ") ["
 		   << token_class << "]" << endl;
+
   if ( filter )
-    word = filter->filter( word );
-  if ( tag == "LET" ){
-    addLemma( sword, UnicodeToUTF8(word) );
+    uword = filter->filter( uword );
+
+  if ( token_class == "ABBREVIATION" ){
+    // We dont handle ABBREVIATION's so just take the word as such
+    string word = UnicodeToUTF8(uword);
+    addLemma( sword, word );
     return;
   }
-  else if ( tag == "SPEC" ){
-    if ( pos.find("eigen") != string::npos ){
-      // SPEC(deeleigen) might contain suffixes
-      if ( token_class == "QUOTE-SUFFIX" ){
-	addLemma( sword, UnicodeToUTF8(UnicodeString( word, 0, word.length()-1)));
-      }
-      else if ( token_class == "WORD-WITHSUFFIX" ){
-	addLemma( sword, UnicodeToUTF8(UnicodeString( word, 0, word.length()-2)));
-      }
-      else {
-	addLemma( sword, UnicodeToUTF8(word) );      return;
-      }
+  auto const& it1 = token_strip_map.find( pos );
+  if ( it1 != token_strip_map.end() ){
+    // some tag/tokenizer_class combinations are special
+    // we have to strip a few letters to get a lemma
+    auto const& it2 = it1->second.find( token_class );
+    if ( it2 != it1->second.end() ){
+      uword = UnicodeString( uword, 0, uword.length() - it2->second );
+      string word = UnicodeToUTF8(uword);
+      addLemma( sword, word );
+      return;
     }
-    else
-      addLemma( sword, UnicodeToUTF8(word) );
+  }
+  if ( one_one_tags.find(pos) != one_one_tags.end() ){
+    // some tags are just taken as such
+    string word = UnicodeToUTF8(uword);
+    addLemma( sword, word );
     return;
   }
-  if ( tag != "SPEC")
-    word.toLower();
-  Classify( word );
+  if ( !keep_case ){
+    uword.toLower();
+  }
+  Classify( uword );
   filterTag( pos );
   makeUnique();
-  getFoLiAResult( sword, word );
+  getFoLiAResult( sword, uword );
 }
 
 void Mblem::Classify( const UnicodeString& uWord ){
@@ -372,19 +371,23 @@ void Mblem::Classify( const UnicodeString& uWord ){
       UnicodeString insstr;
       UnicodeString delstr;
       UnicodeString prefix;
-      for ( size_t l=1; l < edits.size(); ++l ) {
-	switch ( edits[l][0] ){
+      for ( const auto& edit : edits ){
+	if ( edit == edits.front() ){
+	  continue;
+	}
+	switch ( edit[0] ){
 	case 'P':
-	  prefix = UTF8ToUnicode( edits[l].substr( 1 ) );
+	  prefix = UTF8ToUnicode( edit.substr( 1 ) );
 	  break;
 	case 'I':
-	  insstr = UTF8ToUnicode( edits[l].substr( 1 ) );
+	  insstr = UTF8ToUnicode( edit.substr( 1 ) );
 	  break;
 	case 'D':
-	  delstr =  UTF8ToUnicode( edits[l].substr( 1 ) );
+	  delstr =  UTF8ToUnicode( edit.substr( 1 ) );
 	  break;
 	default:
-	  *Log(mblemLog) << "Error: strange value in editstring: " << edits[l] << endl;
+	  *Log(mblemLog) << "Error: strange value in editstring: " << edit
+			 << endl;
 	}
       }
       if (debug){
@@ -447,7 +450,7 @@ void Mblem::Classify( const UnicodeString& uWord ){
     }
     if ( !classMap.empty() ){
       // translate TAG(number) stuf back to CGN things
-      map<string,string>::const_iterator it = classMap.find(restag);
+      auto const& it = classMap.find(restag);
       if ( debug )
 	*Log(mblemLog) << "looking up " << restag << endl;
       if ( it != classMap.end() ){
@@ -464,19 +467,20 @@ void Mblem::Classify( const UnicodeString& uWord ){
     mblemResult.push_back( mblemData( UnicodeToUTF8(lemma), restag ) );
   } // while
   if ( debug ) {
-    *Log(mblemLog) << "stored lemma and tag options: " << mblemResult.size() << " lemma's and " << mblemResult.size() << " tags:" << endl;
-    for( size_t index=0; index < mblemResult.size(); ++index ){
-      *Log(mblemLog) << "lemma alt: " << mblemResult[index].getLemma()
-		     << "\ttag alt: " << mblemResult[index].getTag() << endl;
+    *Log(mblemLog) << "stored lemma and tag options: " << mblemResult.size()
+		   << " lemma's and " << mblemResult.size() << " tags:" << endl;
+    for ( const auto& mbr : mblemResult ){
+      *Log(mblemLog) << "lemma alt: " << mbr.getLemma()
+		     << "\ttag alt: " << mbr.getTag() << endl;
     }
   }
 }
 
 vector<pair<string,string> > Mblem::getResult() const {
   vector<pair<string,string> > result;
-  for ( size_t i=0; i < mblemResult.size(); ++i ){
-    result.push_back( make_pair( mblemResult[i].getLemma(),
-				 mblemResult[i].getTag() ) );
+  for ( const auto& mbr : mblemResult ){
+    result.push_back( make_pair( mbr.getLemma(),
+				 mbr.getTag() ) );
   }
   return result;
 }
