@@ -1,13 +1,13 @@
+/* ex: set tabstop=8 expandtab: */
 /*
-  $Id: mblem_prog.cxx 17992 2015-01-21 14:07:56Z sloot $
-  $URL: https://ilk.uvt.nl/svn/sources/Frog/trunk/src/mblem_prog.cxx $
+  Copyright (c) 2006 - 2016
+  CLST  - Radboud University
+  ILK   - Tilburg University
 
-  Copyright (c) 2006 - 2015
-  Tilburg University
+  This file is part of frog:
 
-  A Morphological-Analyzer for Dutch
-
-  This file is part of frog
+  A Tagger-Lemmatizer-Morphological-Analyzer-Dependency-Parser for
+  several languages
 
   frog is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -23,11 +23,11 @@
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
   For questions and suggestions, see:
-      http://ilk.uvt.nl/software.html
+      https://github.com/LanguageMachines/frog/issues
   or send mail to:
-      timbl@uvt.nl
-*/
+      lamasoftware (at ) science.ru.nl
 
+*/
 
 #include <string>
 #include <iostream>
@@ -40,18 +40,23 @@
 #include "ticcutils/LogStream.h"
 #include "ticcutils/Configuration.h"
 #include "ticcutils/CommandLine.h"
+#include "ticcutils/PrettyPrint.h"
+#include "libfolia/folia.h"
 #include "frog/ucto_tokenizer_mod.h"
 #include "frog/cgn_tagger_mod.h"
 #include "frog/mblem_mod.h"
 
 using namespace std;
 using namespace Timbl;
+using namespace TiCC;
+using namespace Tagger;
 
 LogStream my_default_log( cerr, "", StampMessage ); // fall-back
 LogStream *theErrLog = &my_default_log;  // fill the externals
 
 vector<string> fileNames;
-bool doAll = false;
+bool useTagger = true;
+bool useTokenizer = true;
 
 Configuration configuration;
 static string configDir = string(SYSCONF_PATH) + "/" + PACKAGE + "/";
@@ -65,7 +70,8 @@ void usage( ) {
        << "Options:n" << endl;
   cout << "\t============= INPUT MODE (mandatory, choose one) ========================\n"
        << "\t -t <testfile>    Run mblem on this file\n"
-       << "\t -a               give ALL result. Not just the chosen one(s)\n"
+       << "\t --notokenizer    Don't use a tokenizer, so assume all text is tokenized already.\n"
+       << "\t --notagger       Don't use a tagger to disambiguate, so give ALL variants.\n"
        << "\t -c <filename>    Set configuration file (default " << configFileName << ")\n"
        << "\t============= OTHER OPTIONS ============================================\n"
        << "\t -h. give some help.\n"
@@ -109,7 +115,7 @@ bool parse_args( TiCC::CL_Options& Opts ) {
   };
 
   if ( Opts.extract( 't', value ) ){
-    ifstream is( value.c_str() );
+    ifstream is( value );
     if ( !is ){
       cerr << "input stream " << value << " is not readable" << endl;
       return false;
@@ -119,7 +125,8 @@ bool parse_args( TiCC::CL_Options& Opts ) {
   else {
     fileNames = Opts.getMassOpts();
   };
-  doAll = Opts.is_present( 'a' );
+  useTagger = !Opts.is_present( "notagger" );
+  useTokenizer = !Opts.is_present( "notokenizer" );
   return true;
 }
 
@@ -131,13 +138,17 @@ bool init(){
     cerr << "MBLEM Initialization failed." << endl;
     return false;
   }
-  if ( !tokenizer.init( configuration ) ){
-    cerr << "UCTO Initialization failed." << endl;
-    return false;
+  if ( useTokenizer ){
+    if ( !tokenizer.init( configuration ) ){
+      cerr << "UCTO Initialization failed." << endl;
+      return false;
+    }
   }
-  if ( !tagger.init( configuration ) ){
-    cerr << "CGN Initialization failed." << endl;
-    return false;
+  if ( useTagger ){
+    if ( !tagger.init( configuration ) ){
+      cerr << "Tagger Initialization failed." << endl;
+      return false;
+    }
   }
   cerr << "Initialization done." << endl;
   return true;
@@ -146,22 +157,43 @@ bool init(){
 void Test( istream& in ){
   string line;
   while ( getline( in, line ) ){
-    vector<string> sentences = tokenizer.tokenize( line );
-    for ( size_t s=0; s < sentences.size(); ++s ){
-      vector<TagResult> tagv = tagger.tagLine(sentences[s]);
-      for ( size_t w=0; w < tagv.size(); ++w ){
-	UnicodeString uWord = folia::UTF8ToUnicode(tagv[w].word());
-	myMblem.Classify( uWord );
-	if ( !doAll )
-	  myMblem.filterTag( tagv[w].assignedTag() );
-	vector<pair<string,string> > res = myMblem.getResult();
-	cout << tagv[w].word() << " {" << tagv[w].assignedTag() << "}\t";
-	for ( size_t i=0; i < res.size(); ++i ){
-	  cout << res[i].first << "[" << res[i].second << "]";
-	  if ( i < res.size()-1 )
-	    cout << "/";
+    vector<string> sentences;
+    if ( useTokenizer ){
+      sentences = tokenizer.tokenize( line );
+    }
+    else {
+      sentences.push_back( line );
+    }
+    for ( auto const& s : sentences ){
+      if ( useTagger ){
+	vector<TagResult> tagrv = tagger.tagLine( s );
+	for ( const auto& tr : tagrv ){
+	  UnicodeString uWord = folia::UTF8ToUnicode(tr.word());
+	  myMblem.Classify( uWord );
+	  myMblem.filterTag( tr.assignedTag() );
+	  vector<pair<string,string> > res = myMblem.getResult();
+	  string line = tr.word() + " {" + tr.assignedTag() + "}\t";
+	  for ( const auto& p : res ){
+	    line += p.first + "[" + p.second + "]/";
+	  }
+	  line.erase(line.length()-1);
+	  cout << line << endl;
+	}
+      }
+      else {
+	vector<string> parts;
+	TiCC::split( s, parts );
+	for ( const auto& w : parts ){
+	  UnicodeString uWord = folia::UTF8ToUnicode(w);
+	  myMblem.Classify( uWord );
+	  vector<pair<string,string> > res = myMblem.getResult();
+	  string line = w + "\t";
+	  for ( const auto& p : res ){
+	    line += p.first + "[" + p.second + "]/";
+	  }
+	  line.erase(line.length()-1);
+	  cout << line << endl;
 	}
-	cout << endl;
       }
       cout << "<utt>" << endl << endl;
     }
@@ -173,9 +205,12 @@ void Test( istream& in ){
 
 int main(int argc, char *argv[]) {
   std::ios_base::sync_with_stdio(false);
-  cerr << "mblem " << VERSION << " (c) ILK 1998 - 2015" << endl;
-  cerr << "Induction of Linguistic Knowledge Research Group, Tilburg University" << endl;
-  TiCC::CL_Options Opts("c:t:hVd:a", "version");
+  cerr << "mblem " << VERSION << " (c) CLTS, ILK 2014 - 2016" << endl;
+  cerr << "CLST  - Centre for Language and Speech Technology,"
+       << "Radboud University" << endl
+       << "ILK   - Induction of Linguistic Knowledge Research Group,"
+       << "Tilburg University" << endl;
+  TiCC::CL_Options Opts("c:t:hVd:", "version,notagger,notokenizer");
   try {
     Opts.init(argc, argv);
   }
@@ -190,9 +225,8 @@ int main(int argc, char *argv[]) {
       cerr << "terminated." << endl;
       return EXIT_FAILURE;
     }
-    for ( size_t i=0; i < fileNames.size(); ++i ){
-      string TestFileName = fileNames[i];
-      ifstream in(TestFileName.c_str() );
+    for ( const auto& TestFileName : fileNames ){
+      ifstream in(TestFileName);
       if ( in.good() ){
 	Test( in );
       }
