@@ -1,13 +1,13 @@
+/* ex: set tabstop=8 expandtab: */
 /*
-  $Id: mbma_prog.cxx 17992 2015-01-21 14:07:56Z sloot $
-  $URL: https://ilk.uvt.nl/svn/sources/Frog/trunk/src/mbma_prog.cxx $
+  Copyright (c) 2006 - 2016
+  CLST  - Radboud University
+  ILK   - Tilburg University
 
-  Copyright (c) 2006 - 2015
-  Tilburg University
+  This file is part of frog:
 
-  A Morphological-Analyzer for Dutch
-
-  This file is part of frog
+  A Tagger-Lemmatizer-Morphological-Analyzer-Dependency-Parser for
+  several languages
 
   frog is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -23,11 +23,11 @@
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
   For questions and suggestions, see:
-      http://ilk.uvt.nl/software.html
+      https://github.com/LanguageMachines/frog/issues
   or send mail to:
-      timbl@uvt.nl
-*/
+      lamasoftware (at ) science.ru.nl
 
+*/
 
 #include <string>
 #include <iostream>
@@ -40,18 +40,23 @@
 #include "ticcutils/LogStream.h"
 #include "ticcutils/Configuration.h"
 #include "ticcutils/CommandLine.h"
+#include "ticcutils/PrettyPrint.h"
+#include "libfolia/folia.h"
 #include "frog/ucto_tokenizer_mod.h"
 #include "frog/cgn_tagger_mod.h"
 #include "frog/mbma_mod.h"
 
 using namespace std;
 using namespace Timbl;
+using namespace TiCC;
+using namespace Tagger;
 
 LogStream my_default_log( cerr, "", StampMessage ); // fall-back
 LogStream *theErrLog = &my_default_log;  // fill the externals
 
 vector<string> fileNames;
-bool doAll = false;
+bool useTagger = true;
+bool useTokenizer = true;
 
 Configuration configuration;
 static string configDir = string(SYSCONF_PATH) + "/" + PACKAGE + "/";
@@ -68,7 +73,7 @@ void usage( ) {
        << "\t============= OTHER OPTIONS ============================================\n"
        << "\t -h. give some help.\n"
        << "\t -V or --version .   Show version info.\n"
-       << "\t --daring=<true|false>. Default false\n"
+       << "\t --deep-morph. Deliver structured output. Default false\n"
        << "\t -d <debug level>    (for more verbosity)\n";
 }
 
@@ -105,7 +110,7 @@ bool parse_args( TiCC::CL_Options& Opts ) {
   };
 
   if ( Opts.extract( 't', value ) ){
-    ifstream is( value.c_str() );
+    ifstream is( value );
     if ( !is ){
       cerr << "input stream " << value << " is not readable" << endl;
       return false;
@@ -115,9 +120,10 @@ bool parse_args( TiCC::CL_Options& Opts ) {
   else {
     fileNames = Opts.getMassOpts();
   }
-  doAll = Opts.extract( 'a' );
-  if ( Opts.extract( "daring" ) ){
-    configuration.setatt( "daring", "1", "mbma" );
+  useTagger = !Opts.extract( "notagger" );
+  useTokenizer = !Opts.extract( "notokenizer" );
+  if ( Opts.extract( "deep-morph" ) ){
+    configuration.setatt( "deep-morph", "1", "mbma" );
   };
   return true;
 }
@@ -130,13 +136,17 @@ bool init(){
     cerr << "MBMA Initialization failed." << endl;
     return false;
   }
-  if ( !tokenizer.init( configuration ) ){
-    cerr << "UCTO Initialization failed." << endl;
-    return false;
+  if ( useTokenizer ){
+    if ( !tokenizer.init( configuration ) ){
+      cerr << "UCTO Initialization failed." << endl;
+      return false;
+    }
   }
-  if ( !tagger.init( configuration ) ){
-    cerr << "CGN Initialization failed." << endl;
-    return false;
+  if ( useTagger ){
+    if ( !tagger.init( configuration ) ){
+      cerr << "CGN Initialization failed." << endl;
+      return false;
+    }
   }
   cerr << "Initialization done." << endl;
   return true;
@@ -146,41 +156,68 @@ void Test( istream& in ){
   string line;
   while ( getline( in, line ) ){
     line = TiCC::trim( line );
-    if ( line.empty() )
+    if ( line.empty() ){
       continue;
+    }
     cerr << "processing: " << line << endl;
-    vector<string> sentences = tokenizer.tokenize( line );
-    for ( size_t s=0; s < sentences.size(); ++s ){
-      vector<TagResult> tagv = tagger.tagLine(sentences[s]);
-      for ( size_t w=0; w < tagv.size(); ++w ){
-	UnicodeString uWord = folia::UTF8ToUnicode(tagv[w].word());
-	vector<string> v;
-	size_t num = TiCC::split_at_first_of( tagv[w].assignedTag(),
-					      v, "(,)" );
-	if ( num < 1 ){
-	  throw runtime_error( "error: tag not in right format " );
-	}
-	string head = v[0];
-	if ( head != "SPEC" )
-	  uWord.toLower();
-	myMbma.Classify( uWord );
-	if ( !doAll ){
-	  v.erase(v.begin());
-	  myMbma.filterTag( head, v );
-	}
-	vector<vector<string> > res = myMbma.getResult();
-	cout << tagv[w].word() << " {" << tagv[w].assignedTag() << "}\t";
-	if ( res.size() == 0 ){
-	  cout << "[" << uWord << "]";
-	}
-	else {
-	  for ( size_t i=0; i < res.size(); ++i ){
-	    cout << res[i];
-	    if ( i < res.size()-1 )
-	      cout << "/";
+    vector<string> sentences;
+    if ( useTokenizer ){
+      sentences = tokenizer.tokenize( line );
+    }
+    else {
+      sentences.push_back( line );
+    }
+    for ( auto const& s : sentences ){
+      if ( useTagger ){
+	vector<TagResult> tagv = tagger.tagLine( s );
+	for ( const auto& tr : tagv ){
+	  UnicodeString uWord = folia::UTF8ToUnicode( tr.word() );
+	  vector<string> v;
+	  size_t num = TiCC::split_at_first_of( tr.assignedTag(),
+						v, "(,)" );
+	  if ( num < 1 ){
+	    throw runtime_error( "error: tag not in right format " );
 	  }
+	  string head = v[0];
+	  if ( head != "SPEC" ){
+	    uWord.toLower();
+	  }
+	  myMbma.Classify( uWord );
+	  myMbma.filterHeadTag( head );
+	  myMbma.filterSubTags( v );
+	  vector<string> res = myMbma.getResult();
+	  cout << tr.word() << " {" << tr.assignedTag() << "}\t";
+	  if ( res.size() == 0 ){
+	    cout << "[" << uWord << "]";
+	  }
+	  else {
+	    for ( const auto& r : res ){
+	      cout << r;
+	      if ( &r != &res.back() ){
+		cout << "/";
+	      }
+	    }
+	  }
+	  cout << endl;
+	}
+      }
+      else {
+	vector<string> parts;
+	TiCC::split( s, parts );
+	for ( auto const& w : parts ){
+	  UnicodeString uWord = folia::UTF8ToUnicode(w);
+	  uWord.toLower();
+	  myMbma.Classify( uWord );
+	  vector<string> res = myMbma.getResult();
+	  string line = w + "\t";
+	  for ( auto const& r : res ){
+	    line += r;
+	    if ( &r != &res.back() ){
+	      line += "/";
+	    }
+	  }
+	  cout << line << endl;
 	}
-	cout << endl;
       }
       cout << "<utt>" << endl << endl;
     }
@@ -192,9 +229,12 @@ void Test( istream& in ){
 
 int main(int argc, char *argv[]) {
   std::ios_base::sync_with_stdio(false);
-  cerr << "mbma " << VERSION << " (c) ILK 1998 - 2015" << endl;
-  cerr << "Induction of Linguistic Knowledge Research Group, Tilburg University" << endl;
-  TiCC::CL_Options Opts("aVt:d:hc:","daring,version");
+  cerr << "mbma " << VERSION << " (c) CLTS, ILK 2014 - 2016" << endl;
+  cerr << "CLST  - Centre for Language and Speech Technology,"
+       << "Radboud University" << endl
+       << "ILK   - Induction of Linguistic Knowledge Research Group,"
+       << "Tilburg University" << endl;
+  TiCC::CL_Options Opts("Vt:d:hc:","deep-morph,version,notagger,notokenizer");
   try {
     Opts.init(argc, argv);
   }
@@ -209,9 +249,8 @@ int main(int argc, char *argv[]) {
       cerr << "terminated." << endl;
       return EXIT_FAILURE;
     }
-    for ( size_t i=0; i < fileNames.size(); ++i ){
-      string TestFileName = fileNames[i];
-      ifstream in(TestFileName.c_str() );
+    for ( const auto& TestFileName : fileNames ){
+      ifstream in(TestFileName);
       if ( in.good() ){
 	cerr << "Processing: " << TestFileName << endl;
 	Test( in );
