@@ -1,11 +1,13 @@
+/* ex: set tabstop=8 expandtab: */
 /*
-  $Id: mwu_chunker_mod.cxx 17992 2015-01-21 14:07:56Z sloot $
-  $URL: https://ilk.uvt.nl/svn/sources/Frog/trunk/src/mwu_chunker_mod.cxx $
+  Copyright (c) 2006 - 2016
+  CLST  - Radboud University
+  ILK   - Tilburg University
 
-  Copyright (c) 2006 - 2015
-  Tilburg University
+  This file is part of frog:
 
-  This file is part of frog.
+  A Tagger-Lemmatizer-Morphological-Analyzer-Dependency-Parser for
+  several languages
 
   frog is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -21,9 +23,10 @@
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
   For questions and suggestions, see:
-      http://ilk.uvt.nl/software.html
+      https://github.com/LanguageMachines/frog/issues
   or send mail to:
-      timbl@uvt.nl
+      lamasoftware (at ) science.ru.nl
+
 */
 
 #include <iostream>
@@ -37,27 +40,22 @@
 #include "timbl/TimblAPI.h"
 
 #include "frog/Frog.h" // defines etc.
+#include "ucto/unicode.h"
 #include "frog/mwu_chunker_mod.h"
 
 using namespace std;
 using namespace TiCC;
 using namespace folia;
 
-mwuAna::mwuAna( Word *fwrd ){
+mwuAna::mwuAna( Word *fwrd, const string& txt, const string& glue_tag ){
   spec = false;
-  word = fwrd->str();
+  word = txt;
   string tag;
 #pragma omp critical(foliaupdate)
   {
-    tag = fwrd->annotation<PosAnnotation>()->feat("head");
-  }
-  if ( tag == "SPEC" ){
-#pragma omp critical(foliaupdate)
-    {
-      vector<folia::Feature*> feats = fwrd->select<folia::Feature>( );
-      spec = ( feats.size() == 1 && feats[0]->cls() == "deeleigen" );
-    }
+    tag = fwrd->annotation<PosAnnotation>()->cls();
   }
+  spec = ( tag == glue_tag );
   fwords.push_back( fwrd );
 }
 
@@ -74,7 +72,7 @@ EntitiesLayer *mwuAna::addEntity( const std::string& tagset,
       {
 	KWargs args;
 	args["generate_id"] = sent->id();
-	el = new EntitiesLayer(sent->doc(),args);
+	el = new EntitiesLayer( args, sent->doc() );
 	sent->append( el );
       }
     }
@@ -84,13 +82,13 @@ EntitiesLayer *mwuAna::addEntity( const std::string& tagset,
     Entity *e=0;
 #pragma omp critical(foliaupdate)
     {
-      e = new Entity( el->doc(), args );
+      e = new Entity( args, el->doc() );
       el->append( e );
     }
-    for ( size_t p=0; p < fwords.size(); ++p ){
+    for ( const auto& fw : fwords ){
 #pragma omp critical(foliaupdate)
       {
-	e->append( fwords[p] );
+	e->append( fw );
       }
     }
   }
@@ -99,28 +97,38 @@ EntitiesLayer *mwuAna::addEntity( const std::string& tagset,
 
 Mwu::Mwu(LogStream * logstream){
   mwuLog = new LogStream( logstream, "mwu-" );
+  filter = 0;
 }
 
 Mwu::~Mwu(){
   reset();
   delete mwuLog;
+  delete filter;
 }
 
 void Mwu::reset(){
-  for( size_t i=0; i< mWords.size(); ++i ){
-    delete mWords[i];
+  for ( const auto& it : mWords ){
+    delete it;
   }
   mWords.clear();
 }
 
 void Mwu::add( Word *word ){
-  mWords.push_back( new mwuAna( word ) );
+  UnicodeString tmp;
+#pragma omp critical(foliaupdate)
+  {
+    tmp = word->text();
+  }
+  if ( filter )
+    tmp = filter->filter( tmp );
+  string txt = UnicodeToUTF8( tmp );
+  mWords.push_back( new mwuAna( word, txt, glue_tag ) );
 }
 
 
 bool Mwu::read_mwus( const string& fname) {
   *Log(mwuLog) << "read mwus " + fname << endl;
-  ifstream mwufile(fname.c_str(), ios::in);
+  ifstream mwufile(fname, ios::in);
   if(mwufile.bad()){
     return false;
   }
@@ -165,38 +173,58 @@ bool Mwu::init( const Configuration& config ) {
   if ( val.empty() ){
     version = "1.0";
   }
-  else
+  else {
     version = val;
+  }
   val = config.lookUp( "set", "mwu" );
   if ( val.empty() ){
-    tagset = "http://ilk.uvt.nl/folia/sets/frog-mwu-nl";
+    mwu_tagset = "http://ilk.uvt.nl/folia/sets/frog-mwu-nl";
+  }
+  else {
+    mwu_tagset = val;
+  }
+  string charFile = config.lookUp( "char_filter_file", "tagger" );
+  if ( charFile.empty() )
+    charFile = config.lookUp( "char_filter_file" );
+  if ( !charFile.empty() ){
+    charFile = prefix( config.configDir(), charFile );
+    filter = new Tokenizer::UnicodeFilter();
+    filter->fill( charFile );
+  }
+  val = config.lookUp( "gluetag", "mwu" );
+  if ( val.empty() ){
+    glue_tag = "SPEC(deeleigen)";
+  }
+  else {
+    glue_tag = val;
   }
-  else
-    tagset = val;
 
   return true;
 }
 
-ostream &operator <<( ostream& os,
-		      const Mwu& mwu ){
-  for( size_t i = 0; i < mwu.mWords.size(); ++i )
+ostream &operator<<( ostream& os, const Mwu& mwu ){
+  for ( size_t i = 0; i < mwu.mWords.size(); ++i )
     os << i+1 << "\t" << mwu.mWords[i]->getWord() << endl;
   return os;
 }
 
 void Mwu::addDeclaration( Document& doc ) const {
-  doc.declare( AnnotationType::ENTITY,
-	       tagset,
-	       "annotator='frog-mwu-" + version
-	       + "', annotatortype='auto', datetime='" + getTime() + "'");
+#pragma omp critical(foliaupdate)
+  {
+    doc.declare( AnnotationType::ENTITY,
+		 mwu_tagset,
+		 "annotator='frog-mwu-" + version
+		 + "', annotatortype='auto', datetime='" + getTime() + "'");
+  }
 }
 
 void Mwu::Classify( const vector<Word*>& words ){
-  if ( words.empty() )
+  if ( words.empty() ){
     return;
+  }
   reset();
-  for ( size_t i=0; i < words.size(); ++i ){
-    add( words[i] );
+  for ( const auto& word : words ){
+    add( word );
   }
   Classify();
   EntitiesLayer *el = 0;
@@ -205,20 +233,21 @@ void Mwu::Classify( const vector<Word*>& words ){
   {
     sent = words[0]->sentence();
   }
-  for( size_t i = 0; i < mWords.size(); ++i ){
-    el = mWords[i]->addEntity( tagset, sent, el );
+  for ( const auto& mword : mWords ){
+    el = mword->addEntity( mwu_tagset, sent, el );
   }
 }
 
 void Mwu::Classify(){
-  if ( debug )
+  if ( debug ) {
     *Log(mwuLog) << "Starting mwu Classify" << endl;
+  }
   mymap2::iterator best_match;
   size_t matchLength = 0;
   size_t max = mWords.size();
 
-  // add all current sequences of SPEC(deeleigen) words to MWUs
-  for( size_t i=0; i < max-1; ++i ) {
+  // add all current sequences of the glue_tag words to MWUs
+  for ( size_t i=0; i < max-1; ++i ) {
     if ( mWords[i]->isSpec() && mWords[i+1]->isSpec() ) {
       vector<string> newmwu;
       while ( i < max && mWords[i]->isSpec() ){
@@ -231,34 +260,41 @@ void Mwu::Classify(){
     }
   }
   size_t i;
-  for( i = 0; i < max; i++) {
+  for ( i = 0; i < max; i++) {
     string word = mWords[i]->getWord();
-    if ( debug )
+    if ( debug ){
       *Log(mwuLog) << "checking word[" << i <<"]: " << word << endl;
-    pair<mymap2::iterator, mymap2::iterator> matches = MWUs.equal_range(word);
+    }
+    //    pair<mymap2::iterator, mymap2::iterator> matches = MWUs.equal_range(word);
+    const auto matches = MWUs.equal_range(word);
     if ( matches.first != MWUs.end() ) {
       //match
-      mymap2::iterator current_match = matches.first;
+      auto current_match = matches.first;
       if (  debug  ) {
 	*Log(mwuLog) << "MWU: match found!\t" << current_match->first << endl;
       }
-      while(current_match != matches.second && current_match != MWUs.end()) {
+      while( current_match != matches.second
+	     && current_match != MWUs.end() ){
 	vector<string> match = current_match->second;
 	size_t max_match = match.size();
 	size_t j = 0;
-	if ( debug )
+	if ( debug ){
 	  *Log(mwuLog) << "checking " << max_match << " matches:" << endl;
-	for(; i + j + 1 < max && j < max_match; j++) {
+	}
+	for (; i + j + 1 < max && j < max_match; j++) {
 	  if ( match[j] != mWords[i+j+1]->getWord() ) {
-	    if ( debug )
+	    if ( debug ){
 	      *Log(mwuLog) << "match " << j <<" (" << match[j]
-		   << ") doesn't match with word " << i+ j + 1
+			   << ") doesn't match with word " << i+ j + 1
 			   << " (" << mWords[i+j + 1]->getWord() <<")" << endl;
+	    }
 	    // mismatch in jth word of current mwu
 	    break;
 	  }
-	  else if ( debug )
-	    *Log(mwuLog) << " matched " <<  mWords[i+j+1]->getWord() << " j=" << j << endl;
+	  else if ( debug ){
+	    *Log(mwuLog) << " matched " <<  mWords[i+j+1]->getWord()
+			 << " j=" << j << endl;
+	  }
 
 	}
 	if (j == max_match && j > matchLength ){
@@ -271,14 +307,16 @@ void Mwu::Classify(){
       if( debug ){
 	if (matchLength >0 ) {
 	  *Log(mwuLog) << "MWU: found match starting with " << (*best_match).first << endl;
-	} else {
+	}
+	else {
 	  *Log(mwuLog) <<"MWU: no match" << endl;
 	}
       }
       // we found a matching mwu, break out of loop thru sentence,
       // do useful stuff, and recurse to find more mwus
-      if (matchLength > 0 )
+      if ( matchLength > 0 ){
 	break;
+      }
     } //match found
     else {
       if( debug )
@@ -287,11 +325,13 @@ void Mwu::Classify(){
   } //for (i < max)
   if (matchLength > 0 ) {
     //concat
-    if ( debug )
+    if ( debug ){
       *Log(mwuLog) << "mwu found, processing" << endl;
+    }
     for ( size_t j = 1; j <= matchLength; ++j) {
-      if ( debug )
+      if ( debug ){
 	*Log(mwuLog) << "concat " << mWords[i+j]->getWord() << endl;
+      }
       mWords[i]->merge( mWords[i+j] );
     }
     vector<mwuAna*>::iterator anatmp1 = mWords.begin() + i;
