@@ -1,13 +1,13 @@
+/* ex: set tabstop=8 expandtab: */
 /*
-  $Id: ucto_tokenizer_mod.cxx 17992 2015-01-21 14:07:56Z sloot $
-  $URL: https://ilk.uvt.nl/svn/sources/Frog/trunk/src/ucto_tokenizer_mod.cxx $
+  Copyright (c) 2006 - 2016
+  CLST  - Radboud University
+  ILK   - Tilburg University
 
-  Copyright (c) 2006 - 2015
-  Tilburg University
+  This file is part of frog:
 
-  A Tagger-Lemmatizer-Morphological-Analyzer-Dependency-Parser for Dutch
-
-  This file is part of frog
+  A Tagger-Lemmatizer-Morphological-Analyzer-Dependency-Parser for
+  several languages
 
   frog is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -23,9 +23,10 @@
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
   For questions and suggestions, see:
-      http://ilk.uvt.nl/software.html
+      https://github.com/LanguageMachines/frog/issues
   or send mail to:
-      timbl@uvt.nl
+      lamasoftware (at ) science.ru.nl
+
 */
 
 #include <map>
@@ -33,6 +34,7 @@
 #include "timbl/TimblAPI.h"
 #include "frog/Frog.h"
 #include "ticcutils/Configuration.h"
+#include "ticcutils/FileUtils.h"
 #include "frog/ucto_tokenizer_mod.h"
 
 using namespace std;
@@ -43,6 +45,18 @@ UctoTokenizer::UctoTokenizer(LogStream * logstream) {
   uctoLog = new LogStream( logstream, "tok-" );
 }
 
+string resolve_configdir( const string& rules_name, const string& dir ){
+  if ( TiCC::isFile( rules_name ) ){
+    return rules_name;
+  }
+  string temp = prefix( dir, rules_name );
+  if ( TiCC::isFile( temp ) ){
+    return temp;
+  }
+  // not found. so lets hope ucto can find it!
+  return rules_name;
+}
+
 bool UctoTokenizer::init( const Configuration& config ){
   if ( tokenizer )
     throw runtime_error( "ucto tokenizer is already initialized" );
@@ -55,6 +69,9 @@ bool UctoTokenizer::init( const Configuration& config ){
   }
   if ( !val.empty() )
     debug = TiCC::stringTo<int>( val );
+  if ( !config.hasSection("tokenizer") ){
+    tokenizer->setPassThru();
+  }
   tokenizer->setDebug( debug );
   if ( tokenizer->getPassThru() ){
     // when passthru, we don't further initialize the tokenizer
@@ -67,6 +84,7 @@ bool UctoTokenizer::init( const Configuration& config ){
       return false;
     }
     else {
+      rulesName = resolve_configdir( rulesName, config.configDir() );
       if ( !tokenizer->init( rulesName ) )
 	return false;
     }
@@ -169,14 +187,22 @@ vector<string> UctoTokenizer::tokenize( const string& line ){
     throw runtime_error( "ucto tokenizer not initialized" );
 }
 
-folia::Document UctoTokenizer::tokenize( istream& is ){
+string UctoTokenizer::tokenizeStream( istream& is ){
+  if ( tokenizer ){
+    return tokenizer->tokenizeSentenceStream( is );
+  }
+  else
+    throw runtime_error( "ucto tokenizer not initialized" );
+}
+
+folia::Document *UctoTokenizer::tokenize( istream& is ){
   if ( tokenizer )
     return tokenizer->tokenize( is );
   else
     throw runtime_error( "ucto tokenizer not initialized" );
 }
 
-folia::Document UctoTokenizer::tokenizestring( const string& s){
+folia::Document *UctoTokenizer::tokenizestring( const string& s){
   if ( tokenizer) {
     istringstream is(s);
     return tokenizer->tokenize( is);
